"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path = require("path");
const fs = require("fs");
const os = require("os");
const tmp = require("tmp");
const _ = require("lodash");
const promisify = require("pify");
const AWS = require("aws-sdk");
const _mkdirp = require("mkdirp");
const shelljs = require("shelljs");
const Listr = require("listr");
const prompts_1 = require("./prompts");
const errors_1 = require("./errors");
const validate = tslib_1.__importStar(require("./validate"));
const utils = tslib_1.__importStar(require("./utils"));
const logger_1 = require("./logger");
tmp.setGracefulCleanup();
const mkdirp = promisify(_mkdirp);
const pfs = promisify(fs);
const { prettify, isValidProjectPath, toEnvFile, confirmOrAbort } = utils;
const AWS_CONF_PATH = `${os.homedir()}/.aws/config`;
const getFileNameForItem = item => `${item.id}.json`;
const read = file => fs.readFileSync(file, { encoding: 'utf8' });
const maybeRead = file => {
    if (fs.existsSync(file))
        return read(file);
};
const readJSON = file => JSON.parse(read(file));
const maybeReadJSON = file => {
    const result = maybeRead(file);
    if (result)
        return JSON.parse(result);
};
const write = (file, data) => fs.writeFileSync(file, prettify(data));
const pwrite = (file, data) => pfs.writeFile(file, prettify(data));
const exists = file => fs.existsSync(file);
const getLongFunctionName = ({ stackName, functionName }) => {
    if (functionName.lastIndexOf(stackName) === 0)
        return functionName;
    return `${stackName}-${functionName}`;
};
const readDirOfJSONs = dir => {
    return fs.readdirSync(dir)
        .map(file => require(path.resolve(dir, file)));
};
const normalizeError = err => {
    if (err instanceof Error)
        return err;
    return new Error(JSON.stringify(err));
};
const DEPLOYABLES = [
    'bot',
    'style',
    'models',
    'modelsPack',
    'terms'
];
const getDeployables = opts => _.pick(opts, DEPLOYABLES);
const getDeployablesKeys = opts => Object.keys(getDeployables(opts));
const DEPLOY_ALL_OPTS = DEPLOYABLES.reduce((obj, prop) => {
    obj[prop] = true;
    return obj;
}, {});
const getOptsOnly = opts => _.omit(opts, 'args');
const normalizeDeployOpts = (opts, command = 'deploy') => {
    if (opts.args.length) {
        throw new errors_1.Errors.InvalidInput(`unknown arguments: ${opts.args.join(' ')}`);
    }
    const all = opts.all || !_.size(getDeployables(opts));
    if (all) {
        return _.extend({}, DEPLOY_ALL_OPTS, opts);
    }
    if (!_.size(getDeployables(opts))) {
        throw new errors_1.Errors.InvalidInput(`you didn't indicate anything to ${command}!`);
    }
    return opts;
};
const functions = {
    setconf: 'setconf',
    cli: 'cli',
    importDataUtils: 'import_data_utils'
};
const paths = {
    conf: './conf',
    bot: './conf/bot.json',
    style: './conf/style.json',
    terms: './conf/terms-and-conditions.md',
    models: './models',
    lenses: './lenses'
};
read.bot = () => maybeReadJSON(paths.bot);
read.style = () => maybeReadJSON(paths.style);
read.models = () => readDirOfJSONs(paths.models);
read.lenses = () => readDirOfJSONs(paths.lenses);
read.terms = () => maybeRead(paths.terms);
const createImportDataUtilsMethod = ({ method, props = [], required }) => (data) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    data = _.pick(data, props);
    (required || props).forEach(prop => {
        if (!(prop in data))
            throw new errors_1.Errors.InvalidInput(`expected "${prop}"`);
    });
    return yield this.invokeAndReturn({
        functionName: functions.importDataUtils,
        arg: { method, data }
    });
});
class Conf {
    constructor(opts) {
        this.deploy = (opts) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            const items = this.getDeployItems(opts);
            logger_1.logger.info('deploying: ', getDeployablesKeys(items).join(', '));
            if (opts.dryRun)
                return logger_1.logger.info('dry run, not executing');
            const { error, result } = yield this.invoke({
                functionName: functions.setconf,
                arg: items
            });
            if (error)
                throw error;
            return result;
        });
        this.invoke = (opts) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            let { functionName, arg, local = this.local } = opts;
            if (!functionName)
                functionName = yield prompts_1.fn(this, 'which function?');
            let result;
            try {
                const promise = local ? this._invokeLocal(opts) : this._invoke(opts);
                result = yield promise;
            }
            catch (error) {
                return { error };
            }
            return { result };
        });
        this.invokeAndReturn = (opts) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { error, result } = yield this.invoke(opts);
            if (error)
                throw error;
            return result;
        });
        this.getLongFunctionName = (functionName) => getLongFunctionName({
            stackName: this.stackName,
            functionName
        });
        this.getDeployItems = (opts) => {
            opts = normalizeDeployOpts(opts);
            const parts = {};
            if (opts.style) {
                parts.style = read.style();
            }
            if (opts.terms) {
                parts.terms = read.terms();
            }
            if (opts.models) {
                const models = read.models();
                const lenses = read.lenses();
                if (models.length || lenses.length) {
                    parts.modelsPack = utils.pack({ models, lenses });
                }
            }
            if (opts.bot) {
                parts.bot = read.bot();
            }
            return parts;
        };
        this.load = (opts = {}) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            opts = normalizeDeployOpts(opts, 'load');
            logger_1.logger.info('loading: ', getDeployablesKeys(opts).join(', '));
            if (opts.dryRun)
                return logger_1.logger.info('dry run, not executing');
            const result = yield this.exec({
                args: ['getconf --conf']
            });
            if (opts.style && result.style) {
                logger_1.logger.debug('loaded remote style');
                write(paths.style, result.style);
            }
            if (opts.bot && result.bot) {
                logger_1.logger.debug('loaded remote bot conf');
                write(paths.bot, result.bot);
            }
            if (opts.terms && result.termsAndConditions) {
                logger_1.logger.debug('loaded remote terms and conditions');
                write(paths.terms, result.termsAndConditions.value);
            }
            if (opts.models && result.modelsPack) {
                logger_1.logger.debug('loaded remote models and lenses');
                this.writeModels(result.modelsPack);
            }
        });
        this.writeModels = (modelsPack) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield ['models', 'lenses'].map((prop) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                const arr = modelsPack[prop];
                if (!arr)
                    return;
                yield this.writeToFiles({
                    dir: paths[prop],
                    arr,
                    name: getFileNameForItem
                });
            }));
        });
        this.writeToFiles = ({ dir, name, arr }) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield mkdirp(dir);
            yield Promise.all(arr.map(item => {
                return pwrite(path.join(dir, name(item)), item);
            }));
        });
        this.validate = (opts) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            const items = this.getDeployItems(opts);
            _.each(items, (value, key) => {
                if (!validate[key]) {
                    logger_1.logger.info(`validation for ${key} will be done in cloud-side on deploy`);
                    return;
                }
                if (typeof value !== 'undefined') {
                    logger_1.logger.debug(`validating: ${key}`);
                    validate[key](value);
                }
            });
        });
        this.exec = (opts) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            const res = yield this.invoke({
                functionName: functions.cli,
                arg: opts.args[0]
            });
            let { error, result } = res;
            if (error)
                throw normalizeError(error);
            ({ error, result } = result);
            if (error)
                throw normalizeError(error);
            return result;
        });
        this.init = (opts = {}) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { overwriteEnv, region, awsProfile, stack, projectPath } = yield prompts_1.init(this);
            if (overwriteEnv === false)
                return;
            this.remote = true;
            this.region = region;
            this.profile = awsProfile;
            this.stackName = stack.name;
            this.stackId = stack.id;
            this.client = null;
            const env = {
                region,
                awsProfile,
                stackName: this.stackName,
                stackId: this.stackId
            };
            const tasks = new Listr([
                {
                    title: 'loading deployment info',
                    task: (ctx) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                        ctx.info = yield this.getEndpointInfo();
                    })
                },
                {
                    title: 'initializing local conf',
                    task: (ctx) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                        env.apiBaseUrl = ctx.info.apiBaseUrl;
                        if (projectPath)
                            env.project = path.resolve(process.cwd(), projectPath);
                        write('.env', toEnvFile(env));
                        yield Promise.all([
                            paths.models,
                            paths.lenses,
                            paths.conf
                        ].map(dir => mkdirp(dir)));
                    })
                }
            ]);
            yield tasks.run();
            logger_1.logger.success('initialization complete!');
        });
        this._getAWSClient = (opts = {}) => {
            const { profile = this.profile || process.env.awsProfile, region = this.region } = opts;
            if (region) {
                AWS.config.update({ region });
            }
            if (profile) {
                AWS.config.update({
                    credentials: new AWS.SharedIniFileCredentials({ profile })
                });
            }
            const s3 = new AWS.S3();
            const cloudformation = new AWS.CloudFormation();
            const lambda = new AWS.Lambda();
            return {
                s3,
                cloudformation,
                lambda,
            };
        };
        this.getStacks = (opts = {}) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            const client = this._getAWSClient(opts);
            return yield utils.listStacks(client);
        });
        this.createDataBundle = ({ path }) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            let bundle;
            try {
                bundle = readJSON(path);
            }
            catch (err) {
                throw new errors_1.Errors.InvalidInput('expected "path" to bundle');
            }
            return yield this.invokeAndReturn({
                functionName: functions.importDataUtils,
                arg: {
                    method: 'createbundle',
                    data: bundle
                }
            });
        });
        this.createDataClaim = createImportDataUtilsMethod({
            method: 'createclaim',
            props: ['key', 'claimType']
        });
        this.listDataClaims = createImportDataUtilsMethod({
            method: 'listclaims',
            props: ['key']
        });
        this.getDataBundle = createImportDataUtilsMethod({
            method: 'getbundle',
            props: ['key', 'claimId'],
            required: []
        });
        this._ensureRemote = () => {
            if (this.local) {
                throw new errors_1.Errors.InvalidInput('not supported for local dev env');
            }
        };
        this.destroy = (opts) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            this._ensureStackName();
            this._ensureRemote();
            const { stackName } = this;
            yield confirmOrAbort(`DESTROY REMOTE MYCLOUD ${stackName}?? There's no undo for this one!`);
            yield confirmOrAbort(`Are you REALLY REALLY sure you want to MURDER ${stackName})?`);
            const buckets = yield utils.listStackBucketIds(this.client, stackName);
            buckets.forEach(id => logger_1.logger.info(id));
            yield confirmOrAbort('Delete these buckets?');
            for (const id of buckets) {
                logger_1.logger.info(`emptying and deleting: ${id}`);
                yield utils.destroyBucket(this.client, id);
            }
            yield new Listr([
                {
                    title: 'deleting stack',
                    task: (ctx) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                        logger_1.logger.info('Note: it may take a few minutes for your stack to be deleted');
                        yield utils.deleteStack(this.client, stackName);
                        yield utils.wait(5000);
                        yield utils.awaitStackDelete(this.client, stackName);
                    })
                }
            ])
                .run();
        });
        this.getApiBaseUrl = () => tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (this.local) {
                return null;
            }
            return utils.getApiBaseUrl(this.client, this.stackName);
        });
        this.info = () => tslib_1.__awaiter(this, void 0, void 0, function* () {
            this._ensureStackName();
            return yield this._info();
        });
        this._info = () => tslib_1.__awaiter(this, void 0, void 0, function* () {
            const getLinks = this.invokeAndReturn({
                functionName: 'cli',
                arg: 'links',
                noWarning: true
            });
            const getInfo = this.getEndpointInfo();
            const [links, info] = yield Promise.all([getLinks, getInfo]);
            return Object.assign({ links: links.result }, _.pick(info, ['version', 'chainKey', 'apiBaseUrl']));
        });
        this.getEndpointInfo = () => tslib_1.__awaiter(this, void 0, void 0, function* () {
            const getApiBaseUrl = this.getApiBaseUrl();
            const info = yield this.invokeAndReturn({ functionName: 'info', arg: {} });
            if (info.isBase64Encoded) {
                info.body = new Buffer(info.body, 'base64');
            }
            const endpoint = JSON.parse(info.body);
            endpoint.apiBaseUrl = yield getApiBaseUrl;
            return endpoint;
        });
        this.getFunctions = () => tslib_1.__awaiter(this, void 0, void 0, function* () {
            this._ensureStackName();
            const { client, stackName } = this;
            const functions = yield utils.listStackFunctionIds(client, stackName);
            return functions.map(f => f.slice(stackName.length + 1));
        });
        this.tail = (opts = {}) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.log(Object.assign({ watch: true }, opts));
        });
        this.log = (opts = {}) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            this._ensureStackName();
            this._ensureRemote();
            utils.checkCommandInPath('awslogs');
            const { client, stackName } = this;
            let functionName = opts.args[0];
            if (!functionName) {
                functionName = yield prompts_1.fn(this, 'which one do you want to log?');
            }
            const longName = getLongFunctionName({ stackName, functionName });
            const logOpts = getOptsOnly(opts);
            if (this.profile)
                logOpts.profile = this.profile;
            if (this.region)
                logOpts['aws-region'] = this.region;
            const passThrough = Object.keys(logOpts);
            const logOptsStr = passThrough
                .map(opt => {
                const key = utils.splitCamelCase(opt)
                    .join('-')
                    .toLowerCase();
                const val = logOpts[opt];
                if (val === true) {
                    return `--${key}`;
                }
                return `--${key}=${val}`;
            })
                .join(' ');
            const cmd = `awslogs get /aws/lambda/${longName} ${logOptsStr}`;
            logger_1.logger.info(cmd);
            shelljs.exec(cmd);
        });
        this.balance = () => tslib_1.__awaiter(this, void 0, void 0, function* () {
            return yield this.exec({
                args: ['balance']
            });
        });
        this.disable = () => tslib_1.__awaiter(this, void 0, void 0, function* () {
            return yield this.exec({
                args: ['setenvvar --key DISABLED --value 1']
            });
        });
        this.enable = () => tslib_1.__awaiter(this, void 0, void 0, function* () {
            return yield this.exec({
                args: ['setenvvar --key DISABLED']
            });
        });
        this._ensureStackName = () => {
            if (!this.stackName) {
                throw new errors_1.Errors.InvalidInput(`hm...are you sure you're in the right directory?`);
            }
        };
        this._invoke = ({ functionName, arg = {}, noWarning }) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!(this.remote || noWarning)) {
                yield confirmOrAbort(`Targeting REMOTE deployment. Continue?`);
            }
            this._ensureStackName();
            const { StatusCode, Payload, FunctionError } = yield this.client.lambda.invoke({
                InvocationType: 'RequestResponse',
                FunctionName: this.getLongFunctionName(functionName),
                Payload: JSON.stringify(arg)
            }).promise();
            if (FunctionError || StatusCode >= 300) {
                const message = Payload || FunctionError;
                throw new Error(message.toString());
            }
            return JSON.parse(Payload.toString());
        });
        this._invokeLocal = ({ functionName, arg }) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { project, nodeFlags } = this;
            const flagsStr = Object.keys(nodeFlags)
                .filter(key => nodeFlags[key])
                .map(key => `--${key}`)
                .join(' ');
            if (typeof arg !== 'string')
                arg = JSON.stringify(arg);
            const tmpInput = tmp.fileSync({ postfix: '.json' });
            const tmpOutput = tmp.fileSync({ postfix: '.json' });
            write(tmpInput.name, JSON.stringify(arg));
            const pwd = process.cwd();
            shelljs.cd(project);
            logger_1.logger.debug('be patient, local invocations can be slow');
            const envVars = _.pick(process.env, ['SERVERLESS_OFFLINE_APIGW']);
            if (!envVars.IS_OFFLINE)
                envVars.IS_OFFLINE = '1';
            const command = `${stringifyEnv(envVars)} node ${flagsStr} \
  "${project}/node_modules/.bin/sls" invoke local \
  -f "${functionName}" \
  -l false \
  --path "${tmpInput.name}" \
  --output "${tmpOutput.name}"`;
            logger_1.logger.debug(`running command: ${command}`);
            const result = shelljs.exec(command, { silent: true });
            shelljs.cd(pwd);
            const res = read(tmpOutput.name).trim();
            if (result.code !== 0)
                throw new Error(`invoke failed: ${res || result.stderr}`);
            return res && JSON.parse(res);
        });
        const { region, profile, stackName, local, remote, project, nodeFlags = {} } = opts;
        if (local && remote) {
            throw new errors_1.Errors.InvalidInput('expected "local" or "remote" but not both');
        }
        if (local) {
            if (!project) {
                throw new errors_1.Errors.InvalidInput('expected "project", the path to your local serverless project');
            }
            if (!isValidProjectPath(project)) {
                throw new errors_1.Errors.InvalidInput('expected "project" to point to serverless project dir');
            }
        }
        utils.normalizeNodeFlags(nodeFlags);
        if (remote && !_.isEmpty(nodeFlags)) {
            throw new errors_1.Errors.InvalidInput('node debugging flags are only supported for local operations');
        }
        this.nodeFlags = nodeFlags;
        this.region = region;
        this.profile = profile;
        this.stackName = stackName;
        this.local = local;
        this.remote = remote;
        this.project = project;
        let client;
        Object.defineProperty(this, 'client', {
            set(value) {
                client = value;
            },
            get() {
                if (!client) {
                    client = this._getAWSClient();
                }
                return client;
            }
        });
    }
}
exports.Conf = Conf;
exports.createConf = (opts) => new Conf(opts);
const stringifyEnv = props => Object.keys(props).map(key => `${key}="${props[key]}"`).join(' ');
//# sourceMappingURL=conf.js.map