"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path = require("path");
const fs = require("fs");
const _ = require("lodash");
const shelljs = require("shelljs");
const fetch = require("node-fetch");
const ModelsPack = require("@tradle/models-pack");
const _validateResource = require("@tradle/validate-resource");
const empty_bucket_1 = require("./empty-bucket");
const models_1 = require("./models");
const errors_1 = require("./errors");
const prompts_1 = require("./prompts");
const get = (url) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const res = yield fetch(url);
    if (res.statusCode > 300) {
        throw new Error(res.statusText);
    }
    return yield res.json();
});
exports.get = get;
const getNamespace = ({ models, lenses }) => {
    const model = _.values(models)[0] || _.values(lenses)[0];
    return model && ModelsPack.getNamespace(model.id);
};
exports.getNamespace = getNamespace;
const pack = ({ namespace, models, lenses }) => {
    if (!(_.size(models) || _.size(lenses))) {
        throw new errors_1.Errors.InvalidInput('expected "models" and/or "lenses"');
    }
    if (!namespace) {
        namespace = getNamespace({ models, lenses });
    }
    const modelsPack = ModelsPack.pack({ namespace, models, lenses });
    return modelsPack;
};
exports.pack = pack;
const validateResource = resource => _validateResource({
    models: models_1.models,
    resource
});
exports.validateResource = validateResource;
const prettify = obj => {
    return typeof obj === 'string' ? obj : JSON.stringify(obj, null, 2);
};
exports.prettify = prettify;
const isValidProjectPath = project => {
    return fs.existsSync(path.resolve(project, 'serverless.yml'));
};
exports.isValidProjectPath = isValidProjectPath;
const toEnvFile = obj => Object.keys(obj)
    .map(key => `${key}="${obj[key]}"`)
    .join('\n');
exports.toEnvFile = toEnvFile;
const confirmOrAbort = (msg) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const confirmed = yield prompts_1.confirm(msg);
    if (!confirmed) {
        throw new errors_1.Errors.UserAborted();
    }
});
exports.confirmOrAbort = confirmOrAbort;
const acceptAll = (item) => true;
const listStackResources = (aws, StackName, filter = acceptAll) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    let resources = [];
    const opts = { StackName };
    while (true) {
        let { StackResourceSummaries, NextToken } = yield aws.cloudformation.listStackResources(opts).promise();
        resources = resources.concat(StackResourceSummaries.filter(filter));
        opts.NextToken = NextToken;
        if (!opts.NextToken)
            break;
    }
    return resources;
});
exports.listStackResources = listStackResources;
const listStackResourcesByType = (type, aws, stackName) => {
    return listStackResources(aws, stackName, ({ ResourceType }) => type === ResourceType);
};
const listStackResourceIdsByType = (type, aws, stackName) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const resources = yield listStackResourcesByType(type, aws, stackName);
    return resources.map(r => r.PhysicalResourceId);
});
const listStackBuckets = listStackResourcesByType.bind(null, 'AWS::S3::Bucket');
exports.listStackBuckets = listStackBuckets;
const listStackBucketIds = listStackResourceIdsByType.bind(null, 'AWS::S3::Bucket');
exports.listStackBucketIds = listStackBucketIds;
const listStackFunctions = listStackResourcesByType.bind(null, 'AWS::Lambda::Function');
exports.listStackFunctions = listStackFunctions;
const listStackFunctionIds = listStackResourceIdsByType.bind(null, 'AWS::Lambda::Function');
exports.listStackFunctionIds = listStackFunctionIds;
const destroyBucket = (aws, Bucket) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    yield empty_bucket_1.emptyBucket(aws.s3, Bucket);
    yield aws.s3.deleteBucket({ Bucket }).promise();
});
exports.destroyBucket = destroyBucket;
const deleteStack = (aws, StackName) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    return yield aws.cloudformation.deleteStack({ StackName }).promise();
});
exports.deleteStack = deleteStack;
const awaitStackDelete = (aws, StackName) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    return yield aws.cloudformation.waitFor('stackDeleteComplete', { StackName }).promise();
});
exports.awaitStackDelete = awaitStackDelete;
const listStacks = (aws) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const listStacksOpts = {
        StackStatusFilter: ['CREATE_COMPLETE', 'UPDATE_COMPLETE']
    };
    let stackInfos = [];
    let keepGoing = true;
    while (keepGoing) {
        let { NextToken, StackSummaries } = yield aws.cloudformation.listStacks(listStacksOpts).promise();
        stackInfos = stackInfos.concat(StackSummaries.map(({ StackId, StackName }) => ({
            id: StackId,
            name: StackName
        })));
        listStacksOpts.NextToken = NextToken;
        keepGoing = !!NextToken;
    }
    return stackInfos;
});
exports.listStacks = listStacks;
const getApiBaseUrl = (aws, StackName) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const result = yield aws.cloudformation.describeStacks({ StackName }).promise();
    const { Outputs } = result.Stacks[0];
    const endpoint = Outputs.find(x => /^ServiceEndpoint/.test(x.OutputKey));
    return endpoint.OutputValue;
});
exports.getApiBaseUrl = getApiBaseUrl;
const splitCamelCase = str => str.split(/(?=[A-Z])/g);
exports.splitCamelCase = splitCamelCase;
const checkCommandInPath = cmd => {
    const { code } = shelljs.exec(`command -v ${cmd}`);
    if (code !== 0) {
        throw new errors_1.Errors.InvalidEnvironment(`Please install: ${cmd}`);
    }
};
exports.checkCommandInPath = checkCommandInPath;
const wait = millis => new Promise(resolve => setTimeout(resolve, millis));
exports.wait = wait;
const normalizeNodeFlags = flags => {
    if (!(flags.inspect || flags['inspect-brk'] || flags.debug || flags['debug-brk']))
        return;
    const nodeVersion = Number(process.version.slice(1, 2));
    if (nodeVersion >= 8) {
        if (flags.debug) {
            delete flags.debug;
            flags.inspect = true;
        }
        if (flags['debug-brk']) {
            delete flags['debug-brk'];
            flags['inspect-brk'] = true;
        }
    }
    else {
        if (flags.debug || flags['debug-brk']) {
            flags.inspect = true;
        }
    }
};
exports.normalizeNodeFlags = normalizeNodeFlags;
//# sourceMappingURL=utils.js.map